@@ -83,12 +83,30 @@ double calcOrientation(pointObj *p1, pointObj *p2)
 
 double calcMidAngle(pointObj *p1, pointObj *p2, pointObj *p3)
 {
-  double theta1,theta2;
-  theta1 = atan2(p1->x-p2->x,p1->y-p2->y);
-  if(theta1<0) theta1 += MS_2PI;
-  theta2 = atan2(p3->x-p2->x,p3->y-p2->y);
-  if(theta2<0) theta2 += MS_2PI;
-  return MS_RAD_TO_DEG*((theta1+theta2)/2.0);
+  pointObj p1n;
+  double dx12, dy12, dx23, dy23, l12, l23;
+
+  /* We treat both segments as vector 1-2 and vector 2-3 and 
+   * compute their dx,dy and length 
+   */
+  dx12 = p2->x - p1->x;
+  dy12 = p2->y - p1->y;
+  l12 = sqrt(dx12*dx12 + dy12*dy12);
+  dx23 = p3->x - p2->x;
+  dy23 = p3->y - p2->y;
+  l23 = sqrt(dx23*dx23 + dy23*dy23);
+
+  /* Normalize length of vector 1-2 to same as length of vector 2-3 */
+  if (l12 > 0.0)
+  {
+    p1n.x = p2->x - dx12*(l23/l12);
+    p1n.y = p2->y - dy12*(l23/l12);
+  }
+  else
+    p1n = *p2;  /* segment 1-2 is 0-length, use segment 2-3 for orientation */
+
+  /* Return the orientation defined by the sum of the normalized vectors */
+  return calcOrientation(&p1n, p3);
 }
 
 /*
@@ -96,10 +114,10 @@ double calcMidAngle(pointObj *p1, pointObj *p2, pointObj *p3)
  *  - transform the original shapeobj
  *  - use the styleObj to render the transformed shapeobj
  */
-int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image, shapeObj *shape, styleObj *style, double scalefactor)
+int msDrawTransformedShape(mapObj *map, imageObj *image, shapeObj *shape, styleObj *style, double scalefactor)
 {
   int type = style->_geomtransform.type;
-  int i,j;
+  int i,j,status = MS_SUCCESS;
   switch(type) {
     case MS_GEOMTRANSFORM_END: /*render point on last vertex only*/
       for(j=0; j<shape->numlines; j++) {
@@ -110,7 +128,7 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
         if(style->autoangle==MS_TRUE && line->numpoints>1) {
           style->angle = calcOrientation(&(line->point[line->numpoints-2]),p);
         }
-        msDrawMarkerSymbol(symbolset,image,p,style,scalefactor);
+        status = msDrawMarkerSymbol(map,image,p,style,scalefactor);
       }
       break;
     case MS_GEOMTRANSFORM_START: /*render point on first vertex only*/
@@ -123,7 +141,7 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
         if(style->autoangle==MS_TRUE && line->numpoints>1) {
           style->angle = calcOrientation(p,&(line->point[1]));
         }
-        msDrawMarkerSymbol(symbolset,image,p,style,scalefactor);
+        status = msDrawMarkerSymbol(map,image,p,style,scalefactor);
       }
       break;
     case MS_GEOMTRANSFORM_VERTICES:
@@ -137,7 +155,7 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
           if(style->autoangle==MS_TRUE) {
             style->angle = calcMidAngle(&(line->point[i-1]),&(line->point[i]),&(line->point[i+1]));
           }
-          msDrawMarkerSymbol(symbolset,image,p,style,scalefactor);
+          status = msDrawMarkerSymbol(map,image,p,style,scalefactor);
         }
       }
       break;
@@ -162,14 +180,14 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
                                           (shape->bounds.miny < -padding) ? -padding : shape->bounds.miny;
       bbox_points[1].y=bbox_points[2].y =
                          (shape->bounds.maxy > image->height+padding) ? image->height+padding : shape->bounds.maxy;
-      msDrawShadeSymbol(symbolset, image, &bbox, style, scalefactor);
+      status = msDrawShadeSymbol(map, image, &bbox, style, scalefactor);
     }
     break;
     case MS_GEOMTRANSFORM_CENTROID: {
       double unused; /*used by centroid function*/
       pointObj centroid;
       if(MS_SUCCESS == msGetPolygonCentroid(shape,&centroid,&unused,&unused)) {
-        msDrawMarkerSymbol(symbolset,image,&centroid,style,scalefactor);
+        status = msDrawMarkerSymbol(map,image,&centroid,style,scalefactor);
       }
     }
     break;
@@ -180,6 +198,15 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
 
       p.shape = shape; /* set a few parser globals (hence the lock) */
       p.expr = &(style->_geomtransform);
+
+      if(p.expr->tokens == NULL) { /* this could happen if drawing originates from legend code (#5193) */
+        status = msTokenizeExpression(p.expr, NULL, NULL);
+        if(status != MS_SUCCESS) {
+          msSetError(MS_MISCERR, "Unable to tokenize expression.", "msDrawTransformedShape()");
+          return MS_FAILURE;
+        }
+      }
+
       p.expr->curtoken = p.expr->tokens; /* reset */
       p.type = MS_PARSE_TYPE_SHAPE;
 
@@ -193,10 +220,10 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
       switch (tmpshp->type) {
         case MS_SHAPE_POINT:
         case MS_SHAPE_POLYGON:        
-          msDrawShadeSymbol(symbolset, image, tmpshp, style, scalefactor);
+          status = msDrawShadeSymbol(map, image, tmpshp, style, scalefactor);
           break;
         case MS_SHAPE_LINE:
-          msDrawLineSymbol(symbolset, image, tmpshp, style, scalefactor);        
+          status = msDrawLineSymbol(map, image, tmpshp, style, scalefactor);
           break;
       }
 
@@ -211,7 +238,7 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
       msSetError(MS_MISCERR, "unknown geomtransform", "msDrawTransformedShape()");
       return MS_FAILURE;
   }
-  return MS_SUCCESS;
+  return status;
 }
 
 /*
@@ -221,8 +248,21 @@ int msDrawTransformedShape(mapObj *map, symbolSetObj *symbolset, imageObj *image
 int msGeomTransformShape(mapObj *map, layerObj *layer, shapeObj *shape)
 {
   int i;
-  expressionObj *e =  &layer->_geomtransform;
-  
+  expressionObj *e =  &layer->_geomtransform;  
+
+#ifdef USE_V8_MAPSCRIPT
+  if (!map->v8context) {
+    msV8CreateContext(map);
+    if (!map->v8context)
+    {
+      msSetError(MS_V8ERR, "Unable to create v8 context.", "msGeomTransformShape()");
+      return MS_FAILURE;
+    }
+  }
+
+  msV8ContextSetLayer(map, layer);
+#endif
+ 
   switch(e->type) {
     case MS_GEOMTRANSFORM_EXPRESSION: {
       int status;
