@@ -46,13 +46,14 @@
 
 
 static int is_msIO_initialized = MS_FALSE;
+static int is_msIO_header_enabled = MS_TRUE;
 
 typedef struct msIOContextGroup_t {
   msIOContext stdin_context;
   msIOContext stdout_context;
   msIOContext stderr_context;
 
-  int    thread_id;
+  void*    thread_id;
   struct msIOContextGroup_t *next;
 } msIOContextGroup;
 
@@ -94,7 +95,7 @@ void msIO_Cleanup()
 static msIOContextGroup *msIO_GetContextGroup()
 
 {
-  int nThreadId = msGetThreadId();
+  void* nThreadId = msGetThreadId();
   msIOContextGroup *prev = NULL, *group = io_context_list;
 
   if( group != NULL && group->thread_id == nThreadId )
@@ -145,6 +146,23 @@ static msIOContextGroup *msIO_GetContextGroup()
   return group;
 }
 
+/* returns MS_TRUE if the msIO standard output hasn't been redirected */
+int msIO_isStdContext() {
+  msIOContextGroup *group = io_context_list;
+  void* nThreadId = msGetThreadId();
+  if(!group || group->thread_id != nThreadId) {
+    group = msIO_GetContextGroup();
+    if(!group) {
+      return MS_FALSE; /* probably a bug */
+    }
+  }
+  if(group->stderr_context.cbData == (void*)stderr &&
+      group->stdin_context.cbData == (void*)stdin &&
+      group->stdout_context.cbData == (void*)stdout)
+    return MS_TRUE;
+  return MS_FALSE;
+}
+
 /************************************************************************/
 /*                          msIO_getHandler()                           */
 /************************************************************************/
@@ -152,7 +170,7 @@ static msIOContextGroup *msIO_GetContextGroup()
 msIOContext *msIO_getHandler( FILE * fp )
 
 {
-  int nThreadId = msGetThreadId();
+  void* nThreadId = msGetThreadId();
   msIOContextGroup *group = io_context_list;
 
   msIO_Initialize();
@@ -173,6 +191,19 @@ msIOContext *msIO_getHandler( FILE * fp )
     return NULL;
 }
 
+/************************************************************************/
+/*                      msIO_setHeaderEnabled()                         */
+/************************************************************************/
+
+void msIO_setHeaderEnabled(int bFlag)
+{
+    is_msIO_header_enabled = bFlag;
+}
+
+/************************************************************************/
+/*                           msIO_setHeader()                           */
+/************************************************************************/
+
 void msIO_setHeader (const char *header, const char* value, ...)
 {
   va_list args;
@@ -195,12 +226,15 @@ void msIO_setHeader (const char *header, const char* value, ...)
     }
   } else {
 #endif // MOD_WMS_ENABLED
-    msIO_fprintf(stdout,"%s: ",header);
-    msIO_vfprintf(stdout,value,args);
-    msIO_fprintf(stdout,"\r\n");
+   if( is_msIO_header_enabled ) {
+      msIO_fprintf(stdout,"%s: ",header);
+      msIO_vfprintf(stdout,value,args);
+      msIO_fprintf(stdout,"\r\n");
+   }
 #ifdef MOD_WMS_ENABLED
   }
 #endif
+  va_end( args );
 }
 
 void msIO_sendHeaders ()
@@ -209,8 +243,10 @@ void msIO_sendHeaders ()
   msIOContext *ioctx = msIO_getHandler (stdout);
   if(ioctx && !strcmp(ioctx->label,"apache")) return;
 #endif // !MOD_WMS_ENABLED
-  msIO_printf ("\r\n");
-  fflush (stdout);
+  if( is_msIO_header_enabled ) {
+    msIO_printf ("\r\n");
+    fflush (stdout);
+  }
 }
 
 
@@ -524,79 +560,6 @@ static void msIO_Initialize( void )
   is_msIO_initialized = MS_TRUE;
 }
 
-/* ==================================================================== */
-/* ==================================================================== */
-/*      Stuff related to having a gdIOCtx operate through the msIO      */
-/*      layer.                                                          */
-/* ==================================================================== */
-/* ==================================================================== */
-
-typedef struct {
-#ifdef USE_GD
-  gdIOCtx        gd_io_ctx;
-#endif
-  msIOContext    *ms_io_ctx;
-} msIO_gdIOCtx;
-
-
-#ifdef USE_GD
-/************************************************************************/
-/*                            msIO_gd_putC()                            */
-/*                                                                      */
-/*      A GD IO context callback to put a character, redirected         */
-/*      through the msIO output context.                                */
-/************************************************************************/
-
-static void msIO_gd_putC( gdIOCtx *cbData, int out_char )
-
-{
-  msIO_gdIOCtx *merged_context = (msIO_gdIOCtx *) cbData;
-  char out_char_as_char = (char) out_char;
-
-  msIO_contextWrite( merged_context->ms_io_ctx, &out_char_as_char, 1 );
-}
-
-/************************************************************************/
-/*                           msIO_gd_putBuf()                           */
-/*                                                                      */
-/*      The GD IO context callback to put a buffer of data,             */
-/*      redirected through the msIO output context.                     */
-/************************************************************************/
-
-static int msIO_gd_putBuf( gdIOCtx *cbData, const void *data, int byteCount )
-
-{
-  msIO_gdIOCtx *merged_context = (msIO_gdIOCtx *) cbData;
-
-  return msIO_contextWrite( merged_context->ms_io_ctx, data, byteCount );
-}
-#endif
-
-/************************************************************************/
-/*                          msIO_getGDIOCtx()                           */
-/*                                                                      */
-/*      The returned context should be freed with free() when no        */
-/*      longer needed.                                                  */
-/************************************************************************/
-
-#ifdef USE_GD
-gdIOCtx *msIO_getGDIOCtx( FILE *fp )
-
-{
-  msIO_gdIOCtx *merged_context;
-  msIOContext *context = msIO_getHandler( fp );
-
-  if( context == NULL )
-    return NULL;
-
-  merged_context = (msIO_gdIOCtx *) calloc(1,sizeof(msIO_gdIOCtx));
-  merged_context->gd_io_ctx.putC = msIO_gd_putC;
-  merged_context->gd_io_ctx.putBuf = msIO_gd_putBuf;
-  merged_context->ms_io_ctx = context;
-
-  return (gdIOCtx *) merged_context;
-}
-#endif
 
 /* ==================================================================== */
 /* ==================================================================== */
@@ -723,6 +686,57 @@ void msIO_installStdoutToBuffer()
                         &group->stderr_context );
 }
 
+
+/************************************************************************/
+/*               msIO_pushStdoutToBufferAndGetOldContext()              */
+/*                                                                      */
+/* This function installs a temporary buffer I/O context and returns    */
+/* previously installed stdout handler. This previous stdout handler    */
+/* should later be restored with msIO_restoreOldStdoutContext().        */
+/* This function can be for example used when wanting to ingest into    */
+/* libxml objects XML generated by msIO_fprintf()                       */
+/************************************************************************/
+
+msIOContext* msIO_pushStdoutToBufferAndGetOldContext()
+
+{
+  msIOContextGroup *group = msIO_GetContextGroup();
+  msIOContext *old_context;
+
+  /* Backup current context */
+  old_context = (msIOContext*) msSmallMalloc(sizeof(msIOContext));
+  memcpy(old_context, &group->stdout_context, sizeof(msIOContext));
+
+  msIO_installStdoutToBuffer();
+
+  return old_context;
+}
+
+/************************************************************************/
+/*                    msIO_restoreOldStdoutContext()                    */
+/************************************************************************/
+
+void msIO_restoreOldStdoutContext(msIOContext *context_to_restore)
+{
+  msIOContextGroup *group = msIO_GetContextGroup();
+  msIOContext *prev_context = &group->stdout_context;
+  msIOBuffer* buffer;
+  
+  /* Free memory associated to our temporary context */
+  assert( strcmp(prev_context->label, "buffer") == 0 );
+
+  buffer = (msIOBuffer* )prev_context->cbData;
+  msFree(buffer->data);
+  msFree(buffer);
+
+  /* Restore old context */
+  msIO_installHandlers( &group->stdin_context,
+                        context_to_restore,
+                        &group->stderr_context );
+
+  msFree(context_to_restore);
+}
+
 /************************************************************************/
 /*                    msIO_installStdinFromBuffer()                     */
 /************************************************************************/
@@ -743,6 +757,126 @@ void msIO_installStdinFromBuffer()
                         &group->stderr_context );
 }
 
+/************************************************************************/
+/*              msIO_getAndStripStdoutBufferMimeHeaders()               */
+/*                                                                      */
+/************************************************************************/
+
+hashTableObj* msIO_getAndStripStdoutBufferMimeHeaders()
+{
+  /* -------------------------------------------------------------------- */
+  /*      Find stdout buffer.                                             */
+  /* -------------------------------------------------------------------- */
+  msIOContext *ctx = msIO_getHandler( (FILE *) "stdout" );
+  msIOBuffer  *buf;
+  int start_of_mime_header, current_pos;
+  hashTableObj* hashTable;
+
+  if( ctx == NULL || ctx->write_channel == MS_FALSE
+      || strcmp(ctx->label,"buffer") != 0 ) {
+    msSetError( MS_MISCERR, "Can't identify msIO buffer.",
+                "msIO_getAndStripStdoutBufferMimeHeaders" );
+    return NULL;
+  }
+
+  buf = (msIOBuffer *) ctx->cbData;
+
+  hashTable = msCreateHashTable();
+
+  /* -------------------------------------------------------------------- */
+  /*      Loop over all headers.                                          */
+  /* -------------------------------------------------------------------- */
+  current_pos = 0;
+  while( TRUE ) {
+    int pos_of_column = -1;
+    char* key, *value;
+
+    start_of_mime_header = current_pos;
+    while( current_pos < buf->data_offset )
+    {
+        if( buf->data[current_pos] == '\r' )
+        {
+            if( current_pos + 1 == buf->data_offset ||
+                buf->data[current_pos + 1] != '\n' )
+            {
+                pos_of_column = -1;
+                break;
+            }
+            break;
+        }
+        if( buf->data[current_pos] == ':' )
+        {
+            pos_of_column = current_pos;
+            if( current_pos + 1 == buf->data_offset ||
+                buf->data[current_pos + 1] != ' ' )
+            {
+                pos_of_column = -1;
+                break;
+            }
+        }
+        current_pos++;
+    }
+
+    if( pos_of_column < 0 || current_pos == buf->data_offset ) {
+      msSetError( MS_MISCERR, "Corrupt mime headers.",
+                  "msIO_getAndStripStdoutBufferMimeHeaders" );
+      msFreeHashTable(hashTable);
+      return NULL;
+    }
+
+    key = (char*) malloc( pos_of_column - start_of_mime_header + 1 );
+    memcpy( key, buf->data+start_of_mime_header, pos_of_column - start_of_mime_header);
+    key[pos_of_column - start_of_mime_header] = '\0';
+
+    value = (char*) malloc( current_pos - (pos_of_column+2) + 1 );
+    memcpy( value, buf->data+pos_of_column+2, current_pos - (pos_of_column+2));
+    value[current_pos - (pos_of_column+2)] = '\0';
+
+    msInsertHashTable( hashTable, key, value );
+
+    msFree( key );
+    msFree( value );
+
+    /* -------------------------------------------------------------------- */
+    /*      Go to next line.                                                */
+    /* -------------------------------------------------------------------- */
+    current_pos += 2;
+    if( current_pos == buf->data_offset )
+    {
+      msSetError( MS_MISCERR, "Corrupt mime headers.",
+                  "msIO_getAndStripStdoutBufferMimeHeaders" );
+      msFreeHashTable(hashTable);
+      return NULL;
+    }
+
+    /* If next line is a '\r', this is the end of mime headers. */
+    if( buf->data[current_pos] == '\r' )
+    {
+        current_pos ++;
+        if( current_pos == buf->data_offset ||
+            buf->data[current_pos] != '\n' )
+        {
+            msSetError( MS_MISCERR, "Corrupt mime headers.",
+                        "msIO_getAndStripStdoutBufferMimeHeaders" );
+            msFreeHashTable(hashTable);
+            return NULL;
+        }
+        current_pos ++;
+        break;
+    }
+  }
+
+  /* -------------------------------------------------------------------- */
+  /*      Move data to front of buffer, and reset length.                 */
+  /* -------------------------------------------------------------------- */
+  memmove( buf->data, buf->data+current_pos,
+           buf->data_offset - current_pos );
+  buf->data[buf->data_offset - current_pos] = '\0';
+  buf->data_offset -= current_pos;
+
+  return hashTable;
+}
+
 /************************************************************************/
 /*                 msIO_stripStdoutBufferContentType()                  */
 /*                                                                      */
@@ -920,9 +1054,9 @@ int msIO_bufferWrite( void *cbData, void *data, int byteCount )
   msIOBuffer *buf = (msIOBuffer *) cbData;
 
   /*
-  ** Grow buffer if needed.
+  ** Grow buffer if needed (reserve one extra byte to put nul character)
   */
-  if( buf->data_offset + byteCount > buf->data_len ) {
+  if( buf->data_offset + byteCount >= buf->data_len ) {
     buf->data_len = buf->data_len * 2 + byteCount + 100;
     if( buf->data == NULL )
       buf->data = (unsigned char *) malloc(buf->data_len);
@@ -944,6 +1078,7 @@ int msIO_bufferWrite( void *cbData, void *data, int byteCount )
 
   memcpy( buf->data + buf->data_offset, data, byteCount );
   buf->data_offset += byteCount;
+  buf->data[buf->data_offset] = '\0';
 
   return byteCount;
 }
