@@ -229,7 +229,8 @@ int msLayerOpen(layerObj *layer)
   if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
     layer->connectiontype = MS_TILED_SHAPEFILE;
 
-  if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS)
+  if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS
+      && layer->connectiontype != MS_KERNELDENSITY)
     layer->connectiontype = MS_RASTER;
 
   if ( ! layer->vtable) {
@@ -1564,7 +1565,7 @@ int msLayerApplyPlainFilterToLayer(FilterEncodingNode *psNode, mapObj *map, int
 int msLayerSupportsSorting(layerObj *layer)
 {
   if (layer && (
-         (layer->connectiontype == MS_OGR) || (layer->connectiontype == MS_POSTGIS) || (layer->connectiontype == MS_ORACLESPATIAL)
+    (layer->connectiontype == MS_OGR) || (layer->connectiontype == MS_POSTGIS) || (layer->connectiontype == MS_ORACLESPATIAL) || ((layer->connectiontype == MS_PLUGIN) && (strstr(layer->plugin_library,"msplugin_oracle") != NULL))
                )
      )
     return MS_TRUE;
@@ -1880,7 +1881,8 @@ int msInitializeVirtualTable(layerObj *layer)
   if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
     layer->connectiontype = MS_TILED_SHAPEFILE;
 
-  if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS)
+  if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS
+      && layer->connectiontype != MS_KERNELDENSITY)
     layer->connectiontype = MS_RASTER;
 
   switch(layer->connectiontype) {
@@ -1903,6 +1905,10 @@ int msInitializeVirtualTable(layerObj *layer)
       /* WMS should be treated as a raster layer */
       return(msRASTERLayerInitializeVirtualTable(layer));
       break;
+    case(MS_KERNELDENSITY):
+      /* KERNELDENSITY should be treated as a raster layer */
+      return(msRASTERLayerInitializeVirtualTable(layer));
+      break;
     case(MS_ORACLESPATIAL):
       return(msOracleSpatialLayerInitializeVirtualTable(layer));
       break;
