@@ -193,7 +193,7 @@ static void msRasterLayerInfoInitialize( layerObj *layer )
   /* We need to do this or the layer->layerinfo will be interpreted */
   /* as shapefile access info because the default connectiontype is */
   /* MS_SHAPEFILE. */
-  if (layer->connectiontype != MS_WMS)
+  if (layer->connectiontype != MS_WMS && layer->connectiontype != MS_KERNELDENSITY)
     layer->connectiontype = MS_RASTER;
 
   rlinfo->query_result_hard_max = 1000000;
