@@ -182,7 +182,7 @@ extern "C" {
   } msTiledSHPLayerInfo;
 
   /* shapefileObj function prototypes  */
-  MS_DLL_EXPORT int msShapefileOpen(shapefileObj *shpfile, char *mode, char *filename, int log_failures);
+  MS_DLL_EXPORT int msShapefileOpen(shapefileObj *shpfile, const char *mode, const char *filename, int log_failures);
   MS_DLL_EXPORT int msShapefileCreate(shapefileObj *shpfile, char *filename, int type);
   MS_DLL_EXPORT void msShapefileClose(shapefileObj *shpfile);
   MS_DLL_EXPORT int msShapefileWhichShapes(shapefileObj *shpfile, rectObj rect, int debug);
